/*
 * 页面框架
 * *a>页面跳转，堆栈处理
 * *b>跨页面交互功能封装
 *
 * */

(function(app){
    /*
     * a>窗口、frame堆栈操作
     * ps:主要通过api.config的pathUrl来获取具体路径
     */

    /*
     * 打开新窗口（不建议使用，请使用goWin）
     * param:option打开窗口所需参数(详情查看api)
     * */
    app.openWin = function (option) {
        var opt = $.extend(true, {
            bounces: false,
            vScrollBarEnabled: false,
            hScrollBarEnabled: false,
            allowEdit: true,//暂时全局打开
            slidBackEnabled: option.name.match(/(main)|(root)|(login)|(access)/g) ? false : true,//在主页面之外的页面默认都有滑动返回功能
            animation: {
                type: "none"
            }
        }, option);
        // ios 8.0及以上 使用wk内核
        // if(app.isIos&&app.getVer()>=8){
        //     opt.useWKWebView=true;
        // }
        api.openWin(opt);
        return this;
    };
    /*
     * 打开新frame（不建议使用，请使用goFrame）
     * param：option:打开frame所需参数(详情查看api)
     * */
    app.openFrame = function (option) {
        var opt = $.extend(true, {
            bounces: false,
            vScrollBarEnabled: false,
            hScrollBarEnabled: false,
            slidBackEnabled: false,
            allowEdit: true,//暂时全局打开
            // progress:{
            //     title:"加载中...",
            //     text:"加载..."
            // },
            animation: {
                type: "none"
            }
        }, option);
        /*兼容性处理，rect参数中有字符串转化为数字,否则ios下会抽风*/
        if (app.isObj(opt.rect)) {
            var ary = ["x", "y", "w", "h"], tp, end = {};
            for (var i = 0; i < 4; i++) {
                tp = Number(opt.rect[ary[i]]);
                if (!isNaN(tp)) {
                    end[ary[i]] = tp;
                }
                else {
                    end[ary[i]] = opt.rect[ary[i]];
                }
            }
            opt.rect = end;
        }
        // ios 8.0及以上 使用wk内核
        // if(app.isIos&&app.getVer()>=8){
        //     opt.useWKWebView=true;
        // }
        api.openFrame(opt);
        return this;
    };
   /*
   * 关闭窗口
   * param:option:窗口name/关闭窗口所需参数
   * */
    app.closeWin = function (option) {
        if (option === undefined) {
            option = api.winName;
        }
        if (!app.isObj(option)) {
            option = {name: String(option)};
        }
        var opt = $.extend(true, {
            name: api.winName,
            animation: {
                type: "push",
                subType:"from_left",
                duration:240
            }
        }, option);
        //百度统计模块
        if(api.pageParam&&api.pageParam._category){
            app.tjPageEnd(api.pageParam._category);
        }
        api.closeWin(opt);
        return this;
    };
    /*
    * 关闭多个窗口
    * param:winId:要关闭到的窗口id/或者整个api.closeToWin的参数
    *
    * */
    app.closeToWin=function(p){
        var backParam={
            animation: {
                type: 'none'
            }
        };
        if(typeof p==="string"){
            backParam.name=p;
        }
        else if(app.isObj(p)){
            $.extend(true,backParam,p)
        }
        else {
            return false;
        }
        //百度统计模块
        if(api.pageParam&&api.pageParam._category){
            app.tjPageEnd(api.pageParam._category);
        }
        api.closeToWin(backParam);
    };
    /*
     * 关闭frame
     * param:option: 窗口name/关闭frame所需参数
     * */
    app.closeFrame = function (option) {
        if (option === undefined) {
            option = {name: api.frameName};
        }
        else if (!app.isObj(option)) {
            option = {name: String(option)};
        }
        var opt = $.extend(true, {
            name: api.frameName,
            animation: {
                type: "none"
            }
        }, option);
        api.closeFrame(opt);
        return this;
    };
    /*
     * 隐藏frame
     * param:name:frame name
     * */
    app.hideFrame = function (name) {
        api.setFrameAttr({
            name: name,
            hidden: true
        });
        return this;
    };
    /*
     * 显示frame
     * param：name:frame name animation:动画参数(详见api)
     * */
    app.showFrame = function (name, animation) {
        api.setFrameAttr({
            name: name,
            hidden: false,
            animation: $.extend(true, {
                type: "none"
            }, app.isObj(animation) ? animation : {})
        });
        return this;
    };
    /*
     * 根据id打开新窗口,并传入openWin参数
     * param:id:窗口id(可选)
     *       type:页面名称
     *       param:窗口打开参数
     * ps:param参数除了在原有的api参数上还可以附加以下字段
     *    readyFire:传入一个对象，作为页面刚打开时执行首次fire方法的参数
     *    page:当页面为common页面时，传入page可以代理打开一个子页面
     * ps:打开新窗口时，使用此方法会产生以下影响：
     *    1.传入调用此方法页面的window和frame信息
     *    2.如果传入了id，那么这个页面会以这个值为id
     *    3.否则会为新开的win创建一个name+自增的id，并且如果新开的win是一个代理窗口如(title+frame)，会命名为winName+And+contentName的形式
     * ps:新开页面后，会产生以下影响
     *    1.记录本窗口为最顶层窗口
     *    2.打开新窗口后，会调用一次_initFire参数内的事件
     * */
    app.goWin = function (a,b,c) {
        var _id,_category,_param;
        if(arguments.length>=3){
            _id=a;
            _category=b;
            _param=c;
        }
        else if(arguments.length==2) {
            if(app.isObj(b)){
                _category=a;
                _param=b;
            }
            else if(b!==undefined){
                _id=a;
                _category=b;
            }
        }
        else {
            _category=a;
        }
        //附加上个页面的窗口信息
        var opt = {
            pageParam: {
                _lastPage: {
                    win: api.winName,
                    frame: api.frameName,
                    alone:!api.frameName,//是否是单独的win页面（如果为false，表示上个页面也是frame）
                    category: api.pageParam._category
                },
                _category:_category,
                _initFire: []
            }
        };
        if (app.isObj(_param)) {
            $.extend(true, opt, _param);
        }
        if (app.isArray(opt.readyFire)) {
            opt.pageParam._initFire = opt.readyFire;
        }
        if (app.config.pathUrl[_category]) {
            opt["name"] = _category;
            opt["url"] = app.config.pathUrl[_category];
        }
        else {
            exportError("没有找到指定页面");
        }
        //如果是共用头部，则name为共用头部类型+"And"+子页面id
        if (app.inArray(_category, app.config.commonAry)) {
            if (opt["pageParam"] && opt["pageParam"]["page"]) {
                opt["name"] = _category + "&" + opt["pageParam"]["page"];
                opt["reload"] = true;
            }
            else {
                exportError("没有找到要打开的子页面");
            }
        }
        delete opt.readyFire;
        delete opt.page;
        //如果已传入id，那么使用自定义id
        if(_id){
            opt["name"]=_id;
        }
        //否则加上随机数，以兼容重复打开功能
        else {
            var rp = app.getStorage("repeatPage");
            opt["name"]+="-"+rp++;
            app.setStorage("repeatPage",rp);
        }
        this.openWin(opt);
        return this
    };
    /*
     * 根据id打开新frame,并传入openFrame参数
     * param:id:frame id(可选)
     *       type:页面名称
     *       param:打开frame的参数
     * ps:param参数除了在原有的api参数上还可以附加以下字段
     *    readyFire:传入一个对象，作为页面刚打开时执行首次fire方法的参数
     *    page:当页面为common页面时，传入page可以代理打开一个子页面
     * ps:如果传入了id，那么会以id为名称，否则会以type为名称；
     * ps:一个win上打开多个相同frame，请自定义id
     * */
    app.goFrame = function (a,b,c) {
        var _id,_category,_param;
        if(c!==undefined){
            _id=a;
            _category=b;
            _param=c;
        }
        else {
            _category=a;
            _param=b;
        }
        //附加上个页面的窗口信息
        var opt = {
            pageParam: {
                _lastPage: {
                    win: api.winName,
                    frame: api.frameName,
                    alone:!api.frameName,//是否归属于单独的win页面（如果为false，表示上个页面也是frame）
                    category: api.pageParam._category
                },
                _fireWinMain:app._fireWinMain,
                _category:_category,
                _initFire: []
            }
        };
        if (app.isObj(_param)) {
            $.extend(true, opt, _param);
        }
        if (app.isArray(opt.readyFire)) {
            opt.pageParam._initFire = opt.readyFire;
        }
        //缓存本页面的rect信息
        if (opt.rect) {
            opt.pageParam._rect = opt.rect;
        }
        if (app.config.pathUrl[_category]) {
            opt["name"] = _category;
            opt["url"] = app.config.pathUrl[_category];
        }
        else {
            exportError("没有找到指定页面");
        }
        //如果是共用头部，则name为共用头部类型+"And"+子页面id
        if (app.inArray(_category, app.config.commonAry)) {
            if (opt["pageParam"] && opt["pageParam"]["page"]) {
                opt["name"] = _category + "&" + opt["pageParam"]["page"];
                opt["reload"] = true;
            }
            else {
                exportError("没有找到要打开的子页面,错误页面id为:"+_category);
            }
        }
        delete opt.readyFire;
        delete opt.page;
        //如果已传入id，那么使用自定义id
        if(_id){
            opt["name"]=_id;
        }
        this.openFrame(opt);
        return this
    };

    /*
     * b>go-page,go-back dom自动跳转页面方法
     */
    /*
     * 页面跳转功能
     * param:pageId:目标win name
     *       param:pageParam参数（与goWin不同的是这里只传pageParam）
     * ps:和goWin一样的处理逻辑,只是附加了页面跳转动画参数
     * */
    app.goPage = function () {
        var pa=arguments,sub=arguments.length===3?2:1,opt={
            animation: {
                type: "push",
                subType: "from_right",
                duration: 240
            }
        };
        if(app.isObj(pa[sub])){
            pa[sub]=$.extend(true,opt,{pageParam:pa[sub]});
            if(!app.isObj(pa[sub].pageParam)){
                delete pa[sub].pageParam;
            }
        }
        app.goWin.apply(app,pa);
        return this;
    };

    /*
     * 页面返回功能
     * param:event 可选的: 如果为字符串：如果本页面是由goWin或者goPage打开,那么可以在返回的同时触发一次上页面的自定义事件
     *                    如果为true：强行返回，不验证返回函数
     * */
    app._goBackFn = function () {
        return true;
    };
    app.goBack = function (event, param) {
        var canGoBack = event === true ? true : this._goBackFn();
        if (!!canGoBack) {
            //如果返回了一个数组，那么第一个被定义为返回事件名，第二个被定义为参数
            if (app.isArray(canGoBack) && canGoBack.length === 2 && app.isObj(canGoBack[1])) {
                event = String(canGoBack[0]);
                param = canGoBack[1];
            }
            //如果返回一个字符串，定义为返回事件名
            if (typeof event === "string" && api.pageParam._lastPage) {
                //如果存在_lastPage,则可以触发一次自定义事件和自定义返回回调
                var p = {
                    event: String(event),
                    win: api.pageParam._lastPage.win,
                    param: this.isObj(param) ? param : {}
                };
                if (api.pageParam._lastPage.frame) {
                    p.frame= api.pageParam._lastPage.frame;

                }
                this.fire(p);
            }
            if(api.pageParam._lastPage){
                var p2={
                    name: api.pageParam._lastPage.win,
                    script: "window.app.setBackHere._backHereCallback()"
                };
                if (api.pageParam._lastPage.frame) {
                    p2.frameName= api.pageParam._lastPage.frame;
                }
                api.execScript(p2);
            }
            if (!(app.inArray(api.winName,app.config.basePage))) {
                //除了这三个页面无法返回，其他的都默认Wie关闭窗口
                app.closeWin();
            }
        }
        return this;
    };
    /*
     * 重新配置此页面的返回功能
     * param 调用此方法后，传入的函数将在返回前一步执行，如果函数返回false，页面将不会返回
     * */
    app.resetGoBack = function (fn) {
        if (typeof fn === "function") {
            app._goBackFn = fn
        } else {
            exportError("参数错误")
        }
        return this;
    };

    /*
     * 重载页面上所有go-page、go-back、go-url事件，在html改变之后，要让新节点点击有效，必须再次执行此方法
     * param:selector容器选择器，将会在此容器中进行事件重置
     * */
    app._goPageClickFn = function () {
        window.event.stopPropagation();
        var pageData = $(this).data("param");
        if (typeof pageData === "string") {
            //替换特殊符号为空格避免解析失败
            pageData = pageData.replace(/\s/g, " ");
            try {
                pageData = app.toJson(pageData);
            } catch (e) {}
        }
        try {
            //如果存在id字段，则被用来定义为id
            if(pageData.id){
                app.goPage(pageData.id,pageData.url, pageData.param);
            }
            else {
                app.goPage(pageData.url, pageData.param);
            }
        } catch (e) {}
    };
    app._goBackClickFn = function () {
        window.event.stopPropagation();
        var pageData = $(this).data("param");
        try {
            if (app.isObj(pageData)) {
                app.goBack(pageData.event, pageData.param);
            }
            else {
                app.goBack();
            }
        } catch (e) {
        }
    };
    app._goUrlClickFn = function () {
        window.event.stopPropagation();
        var pageData = $(this).data("url");
        if (pageData !== undefined) {
            app.openOwnBrowser(String(pageData));
        }
        else {
            app.toast("没有找到要跳转的url")
        }
    };
    app.setGo = function (selector) {
        var wp = null;
        if (typeof selector === "string") {
            wp = $("" + selector);
        }
        else {
            wp = $("body");
        }
        var goPageNode = wp.find(".go-page");
        var goBackNode = wp.find(".go-back");
        var goUrlNode = wp.find(".go-url");
        goPageNode.attr("tapmode", "tapmode").each(function () {
            this.onclick = app._goPageClickFn;
        });
        goBackNode.attr("tapmode", "tapmode").each(function () {
            this.onclick = app._goBackClickFn;
        });
        goUrlNode.attr("tapmode", "tapmode").each(function () {
            this.onclick = app._goUrlClickFn;
        });
        api.parseTapmode();
        return this;
    };
    /*
     *  暂停短暂延迟后关闭窗口，期间应用无法操作
     *  param:time:延迟时间 ms
     *        call:关闭前的操作
     * */
    app.closeAndBack=function(time,call){
        app.cover("transparent");
        setTimeout(function(){
            typeof call==="function"&&call();
            app.goBack();
        },app.dataType(time)==="Number"?time:800);
    };
    /*
     * 设置返回到本页面时执行的函数
     * attr: _backHereCallback 执行的回调
     * param:call:回调
     * ps:必须goBack才会调用
     * */
    app.setBackHere =function(call){
        typeof call==="function"&&(app.setBackHere._backHereCallback=call);
        return app;
    };
    app.setBackHere._backHereCallback=function(){};

    /*
     * 手动触发返回目标页面
     * */
    app.triggerBack=function (win,frame){
        if(win){
            var execParam={
                name:win,
                script:"window.app.setBackHere._backHereCallback()"
            };
            if(!!frame){
                execParam.frameName=frame;
            }
            api.execScript(execParam);
        }
    };

    /*
     * b>跨页面执行函数封装
     * */
    /*
     * 监听页面被切换到前台事件
     * attr:_exposeQueue 事件执行队列
     * ps:仅能在win上监听
     * */
    app.listenExpose = function (fn) {
        if(typeof fn === "function"&&app.isWin()){
            if(app.listenExpose._exposeQueue==null){
                app.listenExpose._exposeQueue=[];
                api.addEventListener({
                    name: 'viewappear'
                }, function (ret, err) {
                    app._for(app.listenExpose._exposeQueue,function(i){
                        app.listenExpose._exposeQueue[i]();
                    });
                });
            }
            app.listenExpose._exposeQueue.push(fn);
        }
    };
    /*
     * 全局散播事件
     * */
    app.fireAll = function (event, param) {
        api.sendEvent({
            name: event,
            extra: param || {}
        });
        return app;
    };
    /*
     * 接收全局事件
     * param:event 事件名
     *       fn：回调 arguments:param:散播时传入的参数
     *       extra：针对平台可能出现的某些系统事件可能需要配置一些参数，自定义事件请无视
     * */
    app.listenAll = function (event, fn, extra) {
        var o = {
            name: event
        };
        app.isObj(extra) && (o.extra = extra);
        api.addEventListener(o, function (ret, err) {
            fn(ret, err)
        });
    };

    /*
     * 设置本页面事件独立,重设fireWin主页
     * attr:_fireWinMain:独立标记
     * ps:默认的主窗口为win
     * ps:重设后打开的frame会以本页面为主窗口,所以要在调用此方法后再打开的frame才会认为是在同一环境下
     * */
    app._fireWinMain=null;
    app.setFireWinMain= function(){
        app._fireWinMain={win:api.winName,frame:api.frameName||null};
        return false;
    };
    /*
     * 同窗口内部触发事件
     * param:event:事件名称
     *       child:指定内部子页面执行(不传表示在主页面执行)
     *       param:参数
     * ps:和fire方法可混用，由listen方法共同监听
     * */
    app.fireWin =function (option){
        var opt = $.extend(true,{}, app._fireWinMain?app._fireWinMain:{win:api.winName},{
            event:option.event||null,
            param:option.param||{}
        });
        if(option.child){
            opt.frame=option.child;
        }
        if (opt.event === null || (opt.win === null && opt.frame === null) || !app.isObj(opt.param)) {
            exportError("事件触发参数错误");
        }
        else {
            var p = {};
            opt.win && (p.name = opt.win);
            opt.frame && (p.frameName = opt.frame);
            p.script = "var cacheP=['" + opt.event + "'," + app.toStr(opt.param) + "];if(window.app!==undefined){app.ready(function(){app.trigger(cacheP[0],cacheP[1]);});}" +
                "else{window.appTriggerCache===undefined?(window.appTriggerCache=[cacheP]):(window.appTriggerCache.push(cacheP));}";
            api.execScript(p);
        }
        return this;
    };

    /*
     * 指定页面触发事件
     * param：event（事件名）
     *        win/frame:目标页面的win/frame name（同页面可以不传win name）
     *        param:要传递的参数
     * ps:最好用于同一win内的frame、win之间通信，或有固定id的页面通信
     * */
    app.fire = function (option) {
        var opt = $.extend(true, {
            event: null,
            win: null,
            frame: null,
            param: {}
        }, option);
        if (opt.event === null || (opt.win === null && opt.frame === null) || !app.isObj(opt.param)) {
            exportError("事件触发参数错误");
        }
        else {
            var p = {};
            opt.win && (p.name = opt.win);
            opt.frame && (p.frameName = opt.frame);
            p.script = "var cacheP=['" + opt.event + "'," + app.toStr(opt.param) + "];if(window.app!==undefined){app.ready(function(){app.trigger(cacheP[0],cacheP[1]);});}" +
                "else{window.appTriggerCache===undefined?(window.appTriggerCache=[cacheP]):(window.appTriggerCache.push(cacheP));}";
            api.execScript(p);
        }
        return this;
    };
    /*
     * 跨页面监听事件
     * attr:events
     * param:event（为本页面（frame/win）绑定跨页面事件名）
     *       fn(触发后执行函数)
     * */
    app.listen = function (event, fn) {
        if (typeof event !== "string" || typeof fn !== "function") {
            exportError("事件监听参数错误!");
        }
        if (app.listen.events[event] === undefined) {
            app.listen.events[event] = [];
            app.listen.events[event][0] = fn;
        }
        else {
            app.listen.events[event].push(fn);
        }
        return this;
    };
    app.listen.events = {};
    /*
     * 直接触发本页面绑定的事件(可重复绑定多次，按顺序执行，但是未测试)
     * param:event:要触发的事件
     *       param:手动传入参数
     * */
    app.trigger = function (event, param) {
        if (this.listen.events[event] !== undefined) {
            for (var i = 0; this.listen.events[event] !== undefined ? (i < this.listen.events[event].length) : (false); i++) {
                this.listen.events[event][i](app.isObj(param) ? param : {});
            }
        }
    };
    /*
     * 跨页面事件移除(使用赋值delete)
     * param:event:要移除的事件名
     * */
    app.closeListen = function (event) {
        if (event === undefined) {
            exportError("事件移除参数错误!");
        }
        else if (app.listen.events.hasOwnProperty(event)) {
            if (delete app.listen.events[event] === false) {
                app.listen.events[event] = undefined;
            }
        }
        return this
    };
})(window.app||(window.app={}));