/*
 * 封装具体的页面功能
 * *a>页面数据交互操作
 * *b>注销/客服等业务逻辑操作
 * *c>跨页面数据同步操作
 * */
(function(app){
    /*
     * a>页面数据交互操作
     * */

    /*
     * >ajax提交数据
     * attr:count
     * param:url:你懂得
     *       method:请求方式
     *       data:请求数据
     *       cover:“请求中”，是否显示遮罩，可以设置文字
     *       timeout：最长等待时间 默认10s
     *       callBack：请求有结果后必然执行回调
     *       success: 请求结果成功回调
     *       defeat：请求结果失败回调
     *       after: 请求回调执行完成后额外的回调
     *       token:true/false 是否同时提交用户token
     * ext:内部的参数会强制覆盖param.data
     * return:tag标记,用于取消ajax
     * ps:此方法对本平台接口的数据返回格式进行了验证，并非公用方法
     * */
    app._ajaxCacheTag = [];
    app.ajax = function (option, token, ext) {
        if (option.url === undefined) {
            throw Error("缺少url参数");
        }
        //自增本页面ajax标记
        if (!app.ajax.count) {
            app.ajax.count = 1;
        }
        else {
            app.ajax.count++;
        }
        var opt = {
            method: "get",
            tag: "ajax" + app.ajax.count,
            timeout: 10,
            checkLogin:true,
            data: {}
        };
        //附加token
        if (token) {
            var tkUser = app.getStorage("user");
            if (tkUser.token) {
                opt.data.token = tkUser.token;
            }
            else {
                opt.data.token = "";
                // app.toLogin();
            }
            //所有接口自动附加国家信息
            var countryStg = app.getStorage("countryMsg");
            if (countryStg) {
                opt.data["site_id"] = countryStg.siteId;
            }
            else {
                opt.data["site_id"] = 0;
            }
        }

        var callback, hasMack;
        $.extend(true, opt, option);
        //处理是否打开遮罩
        if (opt.cover) {
            hasMack = true;
            app.loading(typeof opt.cover === "string" ? opt.cover : null);
            delete opt.cover;
        }
        //强制覆盖ext参数
        if(app.isObj(ext)){
            $.extend(true,opt.data,ext);
        }
        var cacheOpt=$.extend(true,{},opt.data);
        //处理type数据封装
        if (opt.method === "post") {
            opt.data = {
                values: opt.data
            }
        }
        else if (opt.method === "file") {
            opt.method = "post";
            opt.data = {
                values: opt.data,
                files: opt.files
            };
            delete opt.files;
        }
        else if (opt.method === "text") {
            opt.method = "post";
            opt.data = {
                body: String(opt.data)
            }
        }
        else {
            opt.method = "get";
            var joinText = "";
            for (var i in opt.data) {
                joinText += "&" + i + "=" + opt.data[i];
            }
            if (opt.url.indexOf(/\?[\S\s]+/) === -1) {
                opt.url += joinText;
            }
            else {
                opt.url += "?" + joinText.slice(1);
            }
            delete opt.data;
        }

        //处理回调
        callback = function (res, e) {
            if (hasMack) {
                var isInCache = app.inArray(opt.tag, app._ajaxCacheTag);
                if (isInCache) {
                    app._ajaxCacheTag.splice(isInCache - 1, 1);
                }
                app.closeLoading();//移除这个tag
            }
            typeof opt.callBack === "function" && opt.callBack(!!res ? res : {}, e);
            if (e) {
                // app.toast("请求失败，状态码：" + e.statusCode + ",错误码：" + e.code + ",错误信息:" + e.msg + ",错误提示：" + e.body);
                var errorData = {statusCode: -2, msg: e.body};
                if (e.code === 1) {
                    errorData.statusCode = 0;
                    errorData.msg = "网络超时，请重试";
                }
                else if (e.code === 0) {
                    errorData.statusCode = 0;
                    errorData.msg = "当前没有网络";
                }
                else {
                    app.toast("网络繁忙，请稍后重试");//其他未知错误自动提示
                }
                typeof opt.defeat === "function" && opt.defeat(errorData,res,cacheOpt);
            }
            else {
                //错误状态
                if (res === "") {
                    app.toast("网络繁忙，请稍后重试");
                }
                else if (res.datas && res.datas["error"]) {
                    //需要登录
                    if (res["need_login"] == "1") {
                            if(opt.checkLogin){
                                app.toast(res.datas["error"]);
                                app.cover("transparent");
                                setTimeout(function () {
                                    app.toLogin();
                                }, 1200);
                                return false;
                            }
                            else {
                                opt.defeat({statusCode: -1, msg: "请登录"},res,cacheOpt);
                                return false;
                            }
                    }
                    if (typeof opt.defeat === "function") {
                        opt.defeat({statusCode: -1, msg: res.datas["error"]},res,cacheOpt);
                    }
                    else {
                        app.toast(res.datas["error"]);
                    }
                }
                else {
                    typeof opt.success === "function" && opt.success(res.datas,res,cacheOpt);
                }
            }
            if (typeof opt.after === "function") {
                opt.after();
            }
        };
        if (typeof opt.before === "function") {
            opt.before();
        }
        api.ajax(opt, callback);
        this._ajaxCacheTag.push(opt.tag);
        return opt.tag;
    };
    /*
     * 取消ajax
     * param：id:app.ajax返回的tag,如果不传 会清楚本页正在执行的所有ajax
     * */
    app.cancelAjax = function (id) {
        if (id) {
            api.cancelAjax({
                tag: id
            });
        }
        else {
            app._for(this._ajaxCacheTag, function (i) {
                api.cancelAjax({
                    tag: app._ajaxCacheTag[i]
                });
            });
            app._ajaxCacheTag = [];
        }
        return this;
    };
    /*
     * file上传
     * param:同ajax
     * ps:method默认为file，其他同ajax #要提交的文件用files对象传入
     * */
    app.file = function (opt, tk ,ext) {
        return app.ajax($.extend(true, {method: "file"}, opt), tk,ext);
    };
    /*
     * post提交ajax
     * param:同ajax
     * ps:method默认为post，其他同ajax
     * */
    app.post = function (opt, tk ,ext) {
        return app.ajax($.extend(true, {method: "post"}, opt), tk,ext);
    };
    /*
     * get上传
     * param:同ajax
     * ps:method默认为get，其他同ajax
     * */
    app.get = function (opt, tk ,ext) {
        return app.ajax($.extend(true, {method: "get"}, opt), tk,ext);
    };

    /*
     * >上拉加载更多和下拉刷新功能
     * ps:此方法对本平台接口的数据返回格式进行了验证，并非公用方法
     * ps:封装了接口规则，所以要求接口要按固定格式返回数据
     */
    /*
     * 设置下拉刷新
     * param(1):obj:下拉样式类型
     *          callback:下拉后执行的函数
     * param(2):callback:下拉后执行的函数
     * */
    app.setRf = function (a, b) {
        var dfOpt = {
            visible: true,//是否可见
//            loadingImg:true,//上拉下拉的图片地址
            bgColor: "#f0f0f0",//背景颜色
            textColor: "#999",//文本颜色
            textDown: "再下拉一点点...",//下拉文字描述
            textUp: "松开以刷新页面",//松开文字描述
            textLoading: "刷新中...",//加载状态描述
            // textTime: "",//最后更新时间
            showTime: false//是否显示更新时间
        };
        var types = {};
        var callBack = null;
        if (typeof a === "string") {
            types[a] !== undefined && $.extend(true, dfOpt, types[a]||{});
            if (typeof b === "function") {
                callBack = b;
            }
        }
        else if (typeof a === "function") {
            callBack = a;
        }
        api.setRefreshHeaderInfo(dfOpt, function (r, e) {
            callBack();
        });
    };
    /*
     * 设置下拉刷新为完成
     * */
    app.stopRf = function () {
        api.refreshHeaderLoadDone();
    };
    /*
     * 设置上拉加载更多(默认不会请求一次,下次请求page会自动加1)
     * attr:_types 请求种类集合 _dfOpt 默认参数
     * param:type:上拉种类（默认为ajax请求,目前仅有ajax请求）
     *       opt:上拉参数
     *       param:ajax参数（如果type为ajax 会给ajax.post传入的参数）
     * ext:强制覆盖的参数
     * */
    app.setLoadMore = function (type, opt, param ,ext) {
        if (!app.isObj(this.setLoadMore._types)) {
            this.setLoadMore._types = {
                "ajax": {}
            };
            this.setLoadMore._dfOpt = {};
        }
        var types = this.setLoadMore._types;
        var dfType = type ? "ajax" : String(type);
        var dfOpt = this.setLoadMore._dfOpt = {//*指可以通过setLoadMoreOpt方法修改的属性
            upAndDown: false,//是否同时设置上拉刷新使用本接口刷新数据
            win: $(window),//视口
            wrapper: "body",//placeholder的容器
            box: "body",//内容容器
            bottom: 0,//距离页面底部距离多少时刷新*
            page: "curpage",//页码关键字
            start: 1,//起始page为1
            pageNum: "page",//列表条数关键字
            num: false,//加载列表条数，false为不传
            loadingText: "加载中，请稍候...",//加载中文字 *
            defeatText: "加载失败，点击重试...",//加载失败文字 *
            // successText:"加载成功",//加载成功文字 *
            successText: "",//加载成功文字 *
            endText: "没有更多数据了",//没有更多数据文字 *
            //首次请求失败显示html
            defeatWrapper: '<div class="nothing" style="padding-top: ' + ((api.frameName !== "" ? api.frameHeight : api.winHeight) / 2 - 135) + 'px;">' +
            '<i class="nothing_icon iconfont icon-networkError"></i><div class="nothing_small">网络异常</div><div class="blank1"></div>' +
            '<div class="nothing_btn"><div class="c_btn" onclick="app.clearLoadMore({cover:true},true);" tapmode>重新加载</div></div></div>',
            // 首次加载无数据显示html*
            nothingWrapper: '<div class="nothing" style="padding-top: ' + ((api.frameName !== "" ? api.frameHeight : api.winHeight) / 2 - 135) + 'px;">' +
            '<i class="nothing_icon iconfont icon-orderBlank"></i><div class="nothing_text">暂无数据</div></div>',
            refreshCallBack: null//下拉刷新时的回调(会提前于ajax前执行)
        };
        if (typeof opt === "string") {
            opt = {box: opt};
        }
        else if (!app.isObj(opt)) {
            throw Error("缺少参数opt！");
        }
        $.extend(true, dfOpt, opt);
        if (dfType === "ajax" && !app.isObj(param)) {
            throw Error("缺少参数ajaxParam！");
        }
        var ajaxParam = param;
        if (!types[dfType]) {
            return false;
        }
        else {
            if (dfType === "ajax") {
                var typesCache = types[dfType];
                typesCache["box"] = $(dfOpt.box).css({
                    "marginBottom": dfOpt.bottom + 30 + "px",
                    "position": "relative"
                });//内容容器
                typesCache["wrapper"] = $(dfOpt.wrapper).css("position", "relative").append('<div id="loadMorePlaceholder"></div>');//placeholder的容器
                typesCache["placeholder"] = $("#loadMorePlaceholder").css({"bottom": "-30px"});//上拉提示
                typesCache["count"] = dfOpt.start;//从第几页开始加载
                typesCache["loadEnd"] = false;//是否加载完所有内容
                //重装回调数据
                typesCache["posting"] = false;//正在请求中标记
                // var successProxy=null,dfProxy=null,callBacProxy=null;
                if (typeof ajaxParam.callBack === "function") {
                    typesCache.callBackProxy = ajaxParam.callBack;
                }
                ajaxParam.callBack = function (data, e) {
                    //请求完成
                    typesCache["placeholder"].text("");
                    typesCache["posting"] = false;
                    //关闭下拉刷新
                    if (dfOpt.upAndDown === -1) {
                        dfOpt.upAndDown = true;
                    }
                    else if (dfOpt.upAndDown) {
                        app.stopRf();
                    }
                    //判断是否还有下一页，如果没有 关闭上拉加载
                    if (!data["hasmore"]) {
                        typesCache["loadEnd"] = true;
                        typesCache["placeholder"].text(dfOpt["endText"]).show();
                    }
                    typesCache.callBackProxy && typesCache.callBackProxy(data, e);
                };
                if (typeof ajaxParam.success === "function") {
                    typesCache.successProxy = ajaxParam.success;
                }
                ajaxParam.success = function (data,res,p) {
                    //请求成功
                    if (!typesCache["loadEnd"]) {
                        typesCache["placeholder"].text(dfOpt["successText"]);
                    }
                    typesCache.successProxy && typesCache.successProxy(data,res,p);
                    //判断第一页是否有数据,如果没有 显示nothingwrapper
                    var htmlContent = typesCache["box"].html().replace(/\s/g, "");
                    if (htmlContent === "") {
                        if (typesCache["count"] === 1 && dfOpt["nothingWrapper"]) {//首次加载没有数据显示html
                            typesCache["box"].html(dfOpt["nothingWrapper"]);
                            api.parseTapmode();
                            typesCache["placeholder"].hide();
                        }
                    }
                    typesCache["count"] += 1;
                };
                if (typeof ajaxParam.defeat === "function") {
                    typesCache.dfProxy = ajaxParam.defeat;
                }
                ajaxParam.defeat = function (data,res,p) {
                    //请求失败 如果是第一页 显示defeatwrapper
                    if (typesCache["count"] === 1 && dfOpt["defeatWrapper"]) {
                        typesCache["box"].html(dfOpt["defeatWrapper"]);
                        api.parseTapmode();
                        typesCache["placeholder"].hide();
                    }//如果不是，显示点击重新加载
                    else if (!typesCache["loadEnd"]) {
                        typesCache["placeholder"].text(dfOpt["defeatText"]).on("click", function () {
                            typesCache["post"]();
                        });
                    }
                    typesCache.dfProxy && typesCache.dfProxy(data,res,p);
                };
                //修正data属性为对象
                if (!app.isObj(ajaxParam.data)) {
                    ajaxParam.data = {};
                }
                //附加消息条数
                if (typeof dfOpt["num"] === "number") {
                    ajaxParam.data[dfOpt.pageNum] = dfOpt["num"];
                }
                typesCache["param"] = ajaxParam;
                typesCache["cacheExt"] = ext;
                ajaxParam = null;
                typesCache["post"] = function (show) {
                    if (typesCache["posting"]) {
                        return false;
                    }
                    if (typesCache["loadEnd"]) {
                        return false;
                    }
                    typesCache["posting"] = true;
                    if (typesCache["count"] === 1) {
                        //清空内容
                        typesCache["box"].html("");
                    }
                    if (show) {
                        typesCache["placeholder"].hide()
                    }
                    else {
                        typesCache["placeholder"].show();
                    }
                    var newOpt = {data: {}};
                    newOpt["data"][dfOpt.page] = typesCache["count"];
                    typesCache["placeholder"].text(dfOpt["loadingText"]).off("click");
                    app.post($.extend(true, {}, typesCache["param"], newOpt), true,typesCache["cacheExt"]);
                };
                //设置滚动事件
                api.addEventListener({
                    name: 'scrolltobottom',
                    extra: {
                        threshold: dfOpt.bottom            //设置距离底部多少距离时触发，默认值为0，数字类型
                    }
                }, function (ret, err) {
                    typesCache["post"]();
                });
                //如果有upAndDown字段，则配置下拉刷新也使用此上拉加载的ajax
                if (dfOpt.upAndDown) {
                    dfOpt.upAndDown = -1;//做个标记，第一次加载没有刷新中状态
                    app.setRf(function () {
                        if (typeof dfOpt.refreshCallBack === "function") {
                            dfOpt.refreshCallBack();
                        }
                        app.clearLoadMore(null, true);
                    });
                }
                //默认请求一次(且本次不需要触发成功关闭刷新中)
                typesCache["post"](true);
            }
        }
        return this;
    };
    /*
     * 修改上拉加载的参数（一些不重要的，立即生效）
     * param:param:上拉加载的参数
     * */
    app.setLoadMoreOpt = function (param) {
        if (this.setLoadMore._dfOpt === undefined) {
            return false;
        }
        var df = this.setLoadMore._dfOpt;
        var aj = this.setLoadMore._types && this.setLoadMore._types["ajax"];
        var newOpt = {};
        if (typeof param["bottom"] === "number") {
            newOpt["bottom"] = param["bottom"];
            $(df.box).css({"marginBottom": newOpt["bottom"] + 30 + "px", "position": "relative"});//内容容器
            aj["placeholder"] = $("#loadMorePlaceholder").css({"bottom": "-30px"});//上拉提示
        }
        typeof param["loadingText"] === "string" && (newOpt["loadingText"] = param["loadingText"]);
        typeof param["defeatText"] === "string" && (newOpt["defeatText"] = param["defeatText"]);
        typeof param["endText"] === "string" && (newOpt["endText"] = param["endText"]);
        if (param["nothingWrapper"] !== undefined) {
            if (typeof param["nothingWrapper"] === "string") {
                newOpt["nothingWrapper"] = param["nothingWrapper"]
            }
            else if (!!param["nothingWrapper"] === false) {
                newOpt["nothingWrapper"] = false;
            }
        }
        if (param["defeatWrapper"] !== undefined) {
            if (typeof param["defeatWrapper"] === "string") {
                newOpt["defeatWrapper"] = param["defeatWrapper"]
            }
            else if (!!param["defeatWrapper"] === false) {
                newOpt["defeatWrapper"] = false;
            }
        }
        $.extend(true, df, newOpt);
        return this;
    };
    /*
     * 静默覆盖上拉加载ajax的参数（加载时才生效的参数）
     * param:data:参数
     *       cover:是否新有的字段完全整体覆盖之前的参数
     *       ext:强制覆盖参数
     * */
    app.resetLoadMore=function (data,cover,ext){
        var ajaxObj = this.setLoadMore && this.setLoadMore._types["ajax"];
        if(ajaxObj && ajaxObj.box){
            if(app.isObj(data)){
                if (app.isObj(data)) {
                    if (cover) {
                        for (var i in data) {
                            if (i === "success") {
                                ajaxObj.successProxy = data[i];
                            }
                            else if (i === "defeat") {
                                ajaxObj.dfProxy = data[i];
                            }
                            else if (i === "callBack") {
                                ajaxObj.callBackProxy = data[i];
                            }
                            else {
                                ajaxObj.param[i] = data[i];
                            }
                        }
                    }
                    else {
                        $.extend(true, ajaxObj.param, data);
                    }
                }
            }
            if(app.isObj(ext)){
                ajaxObj.cacheExt=ext;
            }
        }
        return this
    };
    /*
     * 重置上拉加载，可修改参数，并将页面page重置为0
     * param:data:新的参数
     *       again:是否刷新一次
     *       cover:是否新有的字段完全覆盖之前的参数/覆盖或继承新的参数
     *       ext:强制覆盖参数
     * ps:如果传入true，会默认执行一次，否则相当于清空页面
     * */
    app.clearLoadMore = function (data, again, cover,ext) {
        var ajaxObj = this.setLoadMore && this.setLoadMore._types["ajax"];
        if (ajaxObj && ajaxObj.box) {
            app.resetLoadMore(data,cover,ext);
            ajaxObj.count = 1;
            ajaxObj.loadEnd = false;
            if (again) {
                ajaxObj.post(true);
            }
        }
        return this;
    };
    /*
     * 清除上拉加载
     * */
    app.removeLoadMore = function () {
        var ajaxObj = this.setLoadMore._types["ajax"];
        if (ajaxObj.box) {
            ajaxObj["wrapper"].off("scroll");
            this.setLoadMore._types["ajax"] = null;
        }
        return this;
    };

    /*
    * b>注销/客服等业务逻辑操作
    * */

    /*
    * 检测token是否失效
    * param:token：要检测的token值
    *       call:回调 arguments:0:1/0/-1 未失效/失效/网络问题无法检测
    * ps:仅检测 不做返回登陆操作
    * */
    app.tokenIsDated=function(call){
        var userStg=app.getStorage("user");
        if(typeof userStg.token==="string"){
            app.ajax({
                url: app.config.url + "/index.php?act=index&op=check_token",
                method: "post",
                timeout: 1,
                checkLogin:false,
                data: {token: userStg.token},
                success:function(r,res,param){
                    if(res["need_login"] == "1"){
                        typeof call=="function"&&call(0);
                    }
                    else {
                        typeof call=="function"&&call(1);
                    }
                },
                defeat:function(e){
                    typeof call=="function"&&call(-1);
                }
            });
        }
        else {
            typeof call=="function"&&call(false);
        }
    };


    /*
     * 更新用户数据并检测用户当前token是否已过期（如果已过期会注销）
     * param: rf:检测后是否刷新用户数据(可选)
     *        fn: 回调函数 arguments>state:1:可用 0:失效
     *                               data:更新之后的用户数据
     * */
    app.checkToken = function (rf, fn) {
        var call = function () {
        }, needRf;
        if (typeof rf !== "function") {
            needRf = !!rf;
            if (typeof fn === "function") {
                call = fn;
            }
        }
        else {
            call = rf;
        }
        var userStg = app.getStorage("user");
        if (!userStg.token) {
            call.apply(null, needRf ? [0, {}] : [0]);
        }
        else {
            var opt = {};
            app.ajax({
                url: app.config.url + "/index.php?act=index&op=check_token",
                method: "post",
                timeout: 5,
                data: {token: userStg.token},
                success: function (data) {
                    if (needRf) {
                        opt = {
                            uid: data["member_id"],
                            token: data["token"],
                            username: data["member_name"],//用户昵称
                            mobile: data["mobile"],//手机号
                            avatar: data["member_avatar"],//头像地址
                            areaCode: data["areacode"],//国家时区
                            email: data["member_email"],//邮箱
                            // sex:data["member_sex"]==="0"?"男":"女",//性别
                            areaInfo: data["member_areainfo"],//居住地
                            account: data["account"],//是否已实名认证
                            reg_step: data["reg_step"],//资料完善度
                            member_qq: data["member_qq"],//QQ信息
                            member_wx: data["member_wx"],//微信信息
                            member_type:data["member_type"],//角色：0：B 3:C
                            domain:data["share_domain"]||false,//个人店铺信息
                            inviteNum:data["invite_num"],//个人剩余邀请码
                        };
                        app.setStorage("user", $.extend(true, app.getStorage("user"), opt));
                    }
                    call(1, opt);
                },
                defeat: function () {
                    call.apply(null, needRf ? [0, {}] : [0]);
                }
            });
        }
    };
    /*
    * 检测用户身份
    * 返回值： a:核心代理 b:一般代理 c:普通用户 y:游客
    * */
    app.userIdentity=function(){
        var userStg=app.getStorage("user");
        var key=null;
        if(!userStg.uid){
            key="y";
        }
        else {
            if(userStg["member_type"]==4){
                key="a";
            }
            else if(userStg["member_type"]==0||userStg["member_type"]==1){
                key="b";
            }
            else/*if(userStg["member_type"]==3)*/{
                key="c";
            }
        }
        return key;
    };

    /*
    * 对用户当前身份进行资料完善检测
    * page:register3
    * param: noC:true/false 是否不区分c端进行检测
    *        noJump: true/false 是否不跳转到完善资料
    * 返回值：true 用户当前身份无需完善资料或已完善资料
    *        false 用户需要跳转到完善资料页面进行完善资料
    * */
    app.verifyUserInformation=function(noC,noJump){
        if(noC||this.userIdentity()!="c"){
            var userStorage=app.getStorage("user");
            if(userStorage["reg_step"]!=3){
                if(!noJump){
                    app.goWin("register3",{
                        pageParam:{
                            goMain:false
                        },
                        animation:{
                            type:"push",
                            subType:"from_right"
                        }
                    });
                }
                return false;
            }
        }
        return true
    };

    /*
     * 检测更新
     * module:mam
     * param：call 检测完毕回调 arguments>state:-1,0,1版本过旧/未执行更新/正在更新中
     *                                   msg:正在执行的操作>disabled:更新中,禁止继续使用
     *                                                     closeWidget:应该关闭应用
     *                                                     skip:应该跳过更新
     *                                                     update:后台更新中,可以继续使用
     *                                                     cancel:取消了更新
     *                                                     no:没有需要更新的
     *                                                     error:检测更新报错]
     *        custom 自定义检测方式
     * ps:custom 为了在入口检测强制更新，主页检测可选更新 请分别传入force/auto 参数
     * */
    app.checkUpdate = function (callBack,custom) {
        var mamMd = api.require("mam");
        var call = typeof callBack === "function" ? callBack : function () {
            
        };
        mamMd&&mamMd.checkUpdate(function (r, e) {
            if (r.status) {
                var result = r.result;
                if (result.update) {
                    var updateFn = function () {
                        api.toast({
                            msg: "正在后台下载应用，请稍候...",
                            location: "bottom",
                            global: true
                        });
                        if (!app.isIos) {
                            api.download({
                                url: result.source,
                                report: true
                            }, function (ret, err) {
                                if (ret && 1 == ret.state) {/* 下载完成 */
                                    var savePath = ret.savePath;
                                    api.installApp({
                                        appUri: savePath
                                    });
                                }
                            });
                        }
                        else {
                            api.installApp({
                                appUri: result.source
                            });
                        }
                    };
                    if (result.closed) {//强制更新
                        if(custom==="auto"){
                            return false;//不检测强制更新
                        }
                        api.confirm({
                            title: "新版本发布啦(强制更新):" + result.version,
                            msg: result.updateTip,
                            buttons: ["立即更新", "退出洋姑妈"]
                        }, function (r, e) {
                            var index = r.buttonIndex;
                            if (index == 1) {
                                updateFn();
                                call(1, "disabled");//更新中(禁止使用)
                            }
                            else {
                                api.closeWidget({silent: true});
                                call(-1, "closeWidget");//关闭应用
                            }
                        });
                    }
                    else {//非强制更新
                        if(custom==="force"){
                            return false;//不检测可选更新
                        }
                        //获取上次检测时间,小于一天不提示
                        var lcuStg = parseFloat(app.getStorage("lastCheckUpdate"));
                        var nowTime = new Date().getTime();
                        if (nowTime - lcuStg < 864000000) {
                            call(0, "skip");//跳过更新
                            return false;
                        }
                        api.confirm({
                            title: "新版本发布啦:" + result.version,
                            msg: result.updateTip,
                            buttons: ["立即更新", "稍后再说"]
                        }, function (r, e) {
                            var index = r.buttonIndex;
                            if (index == 1) {
                                updateFn();
                                call(1, "update");//更新中
                            }
                            else if (index == 2) {
                                app.setStorage("lastCheckUpdate", nowTime);
                                call(0, "cancel");//取消了更新
                            }
                            else {
                                call(0, "cancel");
                            }
                        });
                    }
                }
                else {
                    call(0, "no");//没有更新
                }
            }
            else {
                // app.alert(e.msg);
                call(0, "error");//检测更新错误
                // setTimeout(function(){
                //     api.closeWidget({silent:true});
                // },2000)
            }
        });
    };
    /*
     * 注销账号并回到登录页面
     * module:MyImModule
     * */
    app.toLogin = function () {
        //注销ajax回调
        app.cancelAjax();
        //注销客服信息
        app.outIm();
        //通知root页注销所有页面
        setTimeout(function () {
            app.fire({
                event: "goApp",
                win: "root",
                param: {
                    type: "logout"
                }
            });
        }, 360);
        return app;
    };
    
    /*
    * 转化为游客模式访问
    * module:MyImModule
    * */
    app.toVisitor = function () {
        //注销ajax回调
        app.cancelAjax();
        //注销客服信息
        app.outIm();
        //通知root页注销所有页面
        setTimeout(function () {
            app.fire({
                event: "goApp",
                win: "root",
                param: {
                    type: "visitor"
                }
            });
        }, 360);
        return app;
    };

    /*
    * 静默修复完成，提示重启弹窗
    *
    * */
    app.smartAlert=function(v){
        api.alert({
            title: '当前有新的可用更新',
            msg: v,
            buttons:["重启应用"]
        }, function(ret, err) {
            api.rebootApp();
        });
    };

    /*
     * 批量下单缓存操作
     * param：id/[id,...]
     * ps:将id或id数组加入到本地存储quick对象中的orders中
     * */
    app.joinQuickOrder = function (id) {
        var quick = app.getStorage("quick")
        quick.orders.push(id);
        app.setStorage("quick", quick);
        return app;
    };
    /*
     * 重设quick对象中orders
     * param：[id,...]
     */
    app.setQuickOrder = function (ary) {
        if (app.isArray(ary)) {
            app.setStorage("quick", {orders: ary});
        }
        else {
            app.exportError("")
        }
        return app;
    };
    /*
     *获取目前缓存的quick对象
     * */
    app.getQuickOrder = function () {
        var quick = app.getStorage("quick");
        return quick.orders
    };

    /*
    * 调用客服接口
    * */

    /*
    * 初始化客服模块
    * attr:_imMd,_imMd_callBack
    *
    * type:"normal"/"goods"/"order"
    * param: normal:null
    *        goods:txt:简介
    *              image:缩略图
    *              url:商品地址
    *              price:单价
    *        order:txt:简介
    *              url:订单详情地址
    * */
    app.agency._imMd=null;
    app.agency._imMd_callBack=null;
    app.initIm=function(extJson){
        //在客服未初始化之前，禁止使用
        if(!extJson){
            if(!app.getStorage("serverIsInit")){
                this.agency._imMd=null;
                return this;
            }
        }
        this.agency._imMd=api.require("MyImModule");
        if(this.agency._imMd){
            this.agency._imMd_callBack={};
            //如果传入extJson，那么将会注册以下事件
            if(typeof extJson=="object"){
                var param={};
                if(app.isIos){
                    param={appParam:extJson};
                }
                else {
                    param={appParam:app.toStr(extJson)};
                }
                this.agency._imMd.registImNotice(param,function(r,e){
                    var call=app.agency._imMd_callBack;
                    if(r==1){//安卓有新消息
                        if(app.isAndrod){
                            call["newMsg"]&&call["newMsg"](app.agency._imMd);
                        }
                    }
                    else if(r==0){//安卓无新消息
                        if(app.isAndrod){
                            call["noMsg"]&&call["noMsg"](app.agency._imMd);
                        }
                    }
                    else if(r!=0&&r!=1){
                        //ios个推处理im号
                        if(app.isIos){
                            //区分im:前缀的识别为im号
                            var isIm=String(r).match(/^im\:/);
                            if(isIm&&isIm.length&&isIm.index==0){
                                call["server"]&&call["server"](app.agency._imMd,String(r).slice(3));
                                return false;
                            }
                        }
                        call["goUrl"]&&call["goUrl"](app.agency._imMd,r);
                    }
                });
                app.setStorage("serverIsInit",true);
                if(app.isIos){
                    this.agency._imMd.listenState({},function(state){
                        // test(arguments);
                        var call=app.agency._imMd_callBack;
                        if(state==1){
                            call["newMsg"]&&call["newMsg"](app.agency._imMd);
                        }
                        else if(state==0) {
                            call["noMsg"]&&call["noMsg"](app.agency._imMd);
                        }
                    });
                }
            }
            app.initIm=function(){};
        }
        return this;
    };
    app.charIm=function(id,type,p){
        if(!this.agency._imMd){
            this.initIm();
        }
        if(!this.agency._imMd){
            return false;
        }
        var _type,_param,param=app.isObj(p)?p:{};
        //重组参数
        switch (type){
            case "goods":_type="goods";_param={
                txt:param.txt||"",
                image:param.image||"",
                price:param.price||"",
                url:param.url||""
            };break;
            case "order":_type="order";_param={
                txt: param.txt||"",
                image: param.image||"测试",
                price: param.price||"测试",
                buyerName: param.buyerName||"测试",
                buyerPhoneNum: param.buyerPhoneNum||"测试",
                states: param.states||"测试",
                wuliu: param.wuliu||"测试"
            };if(param.url)_param.url=param.url;break;
            default:_type="normal";_param=null;
        }
        this.charImByDevice(id,_type,_param);
    };
    app.closeChar=function(){
        if(this.agency._imMd){
            this.agency._imMd.closeChat(null);
        }
    };
    app.charImByDevice=function(id,type,param){
        var imMd=this.agency._imMd,sendParam;
        var userStg=app.getStorage("user");
        var serviceUser={
            account:userStg.areaCode?userStg.areaCode+"-"+userStg.mobile:userStg.mobile,//账号
            nickname:userStg.username,//昵称
            avatar:userStg.avatar,//头像
        };
        //附加微信、qq信息
        if(userStg.member_wx){
            serviceUser.weixin=userStg.member_wx;
        }
        if(userStg.member_qq){
            serviceUser.qq=userStg.member_qq;
        }
        var serviceParam={
            type:type,
            user:serviceUser
        };
        if(app.isObj(param)){
            serviceParam.data=param;
        }
        if(app.isIos){
            sendParam={
                appParam:{
                    im_number:id,
                    infro:serviceParam
                }
            }
        }
        else {
            sendParam ={appParam:'[ "" ,' + id +',' + app.toStr(serviceParam) +']'};
        }
        imMd["chat"](sendParam);
    };
    app.outIm=function(){
        if(!this.agency._imMd){
            app.initIm();
        }
        var imMd=this.agency._imMd;
        imMd&& imMd.loginOutIM();
    };
    /*
    * 打开客服会话列表
    *
    * */
    app.imOpenList=function(){
        if(!this.agency._imMd){
            this.initIm();
        }
        if(!this.agency._imMd){
            return false;
        }
        var imMd=this.agency._imMd,sendParam;
        var userStg=app.getStorage("user");
        var serviceUser={
            account:userStg.areaCode?userStg.areaCode+"-"+userStg.mobile:userStg.mobile,//账号
            nickname:userStg.username,//昵称
            avatar:userStg.avatar,//头像
        };
        //附加微信、qq信息
        if(userStg.member_wx){
            serviceUser.weixin=userStg.member_wx;
        }
        if(userStg.member_qq){
            serviceUser.qq=userStg.member_qq;
        }
        var serviceParam={
            user:serviceUser
        };
        if(app.isIos){
            sendParam={
                appParam:{
                    infro:serviceParam
                }
            }
        }
        else {
            sendParam ={appParam:'[ "" ,"",' + app.toStr(serviceParam) +']'};
        }
        imMd["chatList"](sendParam);
    };

    /*
    * 回调监听
    * noMsg：没有新消息
    * newMsg：有新消息
    * goUrl:跳转到指定url
    * */
    app.imListen=function(event,call){
        if(!this.agency._imMd) {
            this.initIm();
        }
        if (!this.agency._imMd) {
            return false;
        }
        if(app.isObj(this.agency._imMd_callBack)&&typeof call==="function"){
            switch (event){
                case "noMsg":this.agency._imMd_callBack["noMsg"]=call;break;
                case "newMsg":this.agency._imMd_callBack["newMsg"]=call;break;
                case "goUrl":this.agency._imMd_callBack["goUrl"]=call;break;
                case "server":this.agency._imMd_callBack["server"]=call;break;
            }
        }
    };


    /*
     * 百度统计模块
     * module:baiduMTJ
     * */
    // app.agency._tjMd=null;
    app.initTj=function(){
        var tjMd=api.require("baiduMTJ");
        tjMd&&tjMd.init({}, function(ret, err) {
            if(ret["status"]){
                //...
            }
        });
    };
    app.tjPage=function(pageName){
        var tjMd = api.require('baiduMTJ');
        tjMd&&tjMd["onPageStart"]({
            pagename:pageName,
        }, function(ret, err) {

        });
    };
    app.tjPageEnd=function(pageName){
        var tjMd = api.require('baiduMTJ');
        tjMd&&tjMd["onPageEnd"]({
            pagename:pageName,
        }, function(ret, err) {
        });
    };
    app.tjEvent=function(id,tag){
        var tjMd = api.require('baiduMTJ');
        tjMd["onEvent"]({
            eventid:id,
            label: tag
        }, function(ret, err) {

        });
    };
    /*
     * c>跨页面数据同步操作
     * ps:在本地存储fetchData字段作为数据刷新的判断依据
     *    fetchData={
     *        type:{
     *            local:{
     *                winName:newDate().getTime
     *            },
     *            state:{
     *                time:registerDate().getTime,
     *                ext:{
     *                    //最后一次操作的值
     *                }
     *            }
     *        }
     *    }
     *
     * */

    /*
     * 读取数据
     * param:type:数据种类
     * */
    app.getFetchData = function (type){
        var fetchData=app.getStorage("fetchData");
        return arguments.length?fetchData[type]:fetchData;
    };

    /*
     * 移除数据
     * param:type:数据种类
     * */
    app.removeFetchData = function (type){
        var fetchData=app.getStorage("fetchData");
        if(typeof type === "string"&&fetchData[type]){
            delete fetchData[type];
            app.setStorage(fetchData);
        }
        return app
    };

    /*
     * 清空数据
     * */
    app.clearFetchData =function (){
        app.setStorage("fetchData",{});
        return app
    };

    /*
     * 为本页面注册一条本地数据监听(同一win内都无法重复注册，再次注册会被覆盖)
     * param: type:数据种类
     *        call:更新变动回调操作 arguments>type:变动类型(目前只有refresh)
     *                                       ext:上次变动的数据
     * */
    app._localFetchCall={};
    app.registerFetch = function (type,call) {
        if(type&&typeof type === "string"){
            var fetchData=app.getStorage("fetchData");
            if(!fetchData[type]){
                fetchData[type]={
                    local:{},
                    state:{
                        time:0,
                        ext:null
                    }
                };
            }
            fetchData[type].local[api.winName]=fetchData[type].state.time;
            if(typeof call==="function"){
                app._localFetchCall[type]=call;
            }
            app.setStorage("fetchData",fetchData);
        }
        else {
            exportError("数据异常");
        }
        return app;
    };

    /*
     * 触发一次本地数据更新
     * param: type:数据种类
     *        data:存入变动数据
     * ps:本页面绑定的监听不会调用，如果不想下次进入此页面重复刷新一次，在本页面再次调用lostFetch方法
     * */
    app.updateFetch = function (type,data){
        var fetchData=app.getStorage("fetchData");
        if(fetchData[type]){
            fetchData[type].state={
                time:new Date().getTime(),
                ext:data
            };
            app.setStorage("fetchData",fetchData);
        }
        return app;
    };

    /*
     * 手动检测一次本地是否是最新数据
     * param: type:数据种类(可选)
     * */
    app.examineFetchHandle=function (type){
        var fetchData=app.getStorage("fetchData");
        var localTime,fetchTime,mack;
        if(typeof type==="string"&&fetchData[type]){
            fetchTime=fetchData[type].state.time;
            localTime=fetchData[type].local[api.winName];
            if(localTime!==fetchTime){
                //与本地时间不一致，触发fireAll
                app.fireAll("fetchDataChange-"+api.winName,{type:type,data:fetchData[type].state.ext});
                fetchData[type].local[api.winName]=fetchTime;
                mack=true;
            }
        }
        else {
            app._each(fetchData,function(i){
                if(fetchData[i].local[api.winName]!==undefined&&fetchData[i].state.time!==fetchData[i].local[api.winName]){
                    //与本地时间不一致，触发fireAll
                    app.fireAll("fetchDataChange-"+api.winName,{type:i,data:fetchData[i].state.ext});
                    fetchData[i].local[api.winName]=fetchData[i].state.time;
                    mack=true;
                }
            });
        }
        mack&&app.setStorage("fetchData",fetchData);
        return app;
    };
    /*
     * 跳过指定页面的本次更新检测
     * param:type:数据种类
     *       id:指定页面id(不传代表当前窗口)
     * */
    app.loseFetch = function (type,id){
        var tId=id?id: api.winName;
        var fetchData=app.getStorage("fetchData");
        if(typeof type==="string"&&fetchData[type]&&fetchData[type].local[tId]){
            fetchData[type].local[tId]=fetchData[type].state.time;
        }
        return app;
    }

})(window.app||(window.app={}));